"""DNS Authenticator for Aliyun DNS."""

import os
import logging
from typing import Callable, Optional
from alibabacloud_alidns20150109.client import Client as Alidns20150109Client
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_alidns20150109 import models as alidns_20150109_models

from certbot.plugins import dns_common
from certbot import errors

logger = logging.getLogger(__name__)


class Authenticator(dns_common.DNSAuthenticator):
    """DNS Authenticator for Aliyun DNS

    This Authenticator uses the Aliyun DNS API to fulfill a dns-01 challenge.
    """

    description = "Obtain certificates using a DNS TXT record (if you are using Aliyun DNS)."
    ttl = 600
    _alidns_client = None

    def __init__(self, *args, **kwargs):
        super(Authenticator, self).__init__(*args, **kwargs)
        self.credentials: Optional[dns_common.CredentialsConfiguration] = None

    @classmethod
    def add_parser_arguments(
        cls, add: Callable[..., None], default_propagation_seconds: int = 10
    ) -> None:  # pylint: disable=arguments-differ
        super().add_parser_arguments(add, default_propagation_seconds)
        add("credentials", help="Aliyun credentials INI file.")

    def more_info(self):  # pylint: disable=missing-docstring,no-self-use
        return "This plugin configures a DNS TXT record to respond to a dns-01 challenge using " + "the Aliyun DNS API."

    def _validate_credentials(self, credentials: dns_common.CredentialsConfiguration) -> None:
        key_id = credentials.conf("key-id")
        key_secret = credentials.conf("key-secret")
        if not key_id or not key_secret:
            raise errors.PluginError(
                "{}: dns_aliyun_key_id and dns_aliyun_key_secret are required.".format(credentials.confobj.filename)
            )

    def _setup_credentials(self) -> None:
        self.credentials = self._configure_credentials(
            "credentials",
            "Aliyun credentials INI file",
            None,
            self._validate_credentials,
        )

    def _perform(self, domain, validation_name, validation):
        domain = self._find_domain_name(domain)
        rr = validation_name[: validation_name.rindex("." + domain)]
        self._get_alidns_client().add_domain_record(
            alidns_20150109_models.AddDomainRecordRequest(
                domain_name=domain,
                rr=rr,
                type="TXT",
                value=validation,
                ttl=self.ttl,
            )
        )

    def _cleanup(self, domain, validation_name, validation):
        domain = self._find_domain_name(domain)
        rr = validation_name[: validation_name.rindex("." + domain)]
        record_id = self._find_domain_record_id(domain, rr=rr, typ="TXT")
        self._get_alidns_client().delete_domain_record(
            alidns_20150109_models.DeleteDomainRecordRequest(record_id=record_id)
        )

    def _get_alidns_client(self):
        if not self._alidns_client:
            if not self.credentials:
                raise Exception("No credentials")
            key_id = self.credentials.conf("key-id")
            key_secret = self.credentials.conf("key-secret")
            if not key_id or not key_secret:
                raise Exception("No key-id or key-secret")
            self._alidns_client = Alidns20150109Client(
                open_api_models.Config(
                    access_key_id=key_id,
                    access_key_secret=key_secret,
                    # Endpoint 请参考 https://api.aliyun.com/product/Alidns
                    endpoint="alidns.cn-hangzhou.aliyuncs.com",
                )
            )
        return self._alidns_client

    def _find_domain_name(self, domain):
        domain_name_guesses = dns_common.base_domain_name_guesses(domain)
        for domain_name in domain_name_guesses:
            response = self._get_alidns_client().describe_domains(
                alidns_20150109_models.DescribeDomainsRequest(key_word=domain_name)
            )
            for d in response.body.domains.domain:
                if d.domain_name == domain_name:
                    return domain_name
        raise errors.PluginError(
            "Unable to determine zone identifier for {0} using zone names: {1}".format(domain, domain_name_guesses)
        )

    def _find_domain_record_id(self, domain, rr="", typ="", value="") -> str:
        for r in (
            self._get_alidns_client()
            .describe_domain_records(
                alidns_20150109_models.DescribeDomainRecordsRequest(
                    domain_name=domain,
                    rrkey_word=rr,
                    type_key_word=typ,
                    value_key_word=value,
                )
            )
            .body.domain_records.record
        ):
            if r.rr == rr and isinstance(r.record_id, str):
                return r.record_id
        raise errors.PluginError(
            "Unexpected error determining record identifier for {0}: {1}".format(rr, "record not found")
        )



import os, sys
from pathlib import Path
from typing import List, TextIO
import rich_click as click
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.console import Group
from datetime import datetime, timezone
import logging
import dateutil.parser
from configobj import ConfigObj

from .cert import Aliyun

cprint = Console().print

log = logging.getLogger()
formatter = logging.Formatter("%(asctime)s - %(message)s")
sh = logging.StreamHandler(sys.stdout)
sh.setFormatter(formatter)
log.setLevel(logging.INFO)
log.addHandler(sh)

pass_aliyun = click.make_pass_decorator(Aliyun)


class RenewedDomains(click.ParamType):
    name = "renewed_domains"

    def convert(self, value, param, ctx) -> List[str]:
        if value and isinstance(value, str):
            return value.strip().split(" ")
        return []


@click.group()
@click.option("--access-key-id", envvar="ALIYUN_ACCESS_KEY_ID", help="Aliyun access key id")
@click.option(
    "--access-key-secret",
    envvar="ALIYUN_ACCESS_KEY_SECRET",
    help="Aliyun access key secret",
)
@click.option(
    "--access-key-ini-file",
    envvar="ALIYUN_ACCESS_KEY_INI_FILE",
    default=str(Path.home() / ".secrets/aliyun.ini"),
    help="Aliyun access key ini file",
)
@click.pass_context
def cli(ctx, access_key_id: str, access_key_secret: str, access_key_ini_file: str) -> None:
    if access_key_id and access_key_secret:
        ctx.obj = Aliyun(access_key_id, access_key_secret)
    elif access_key_ini_file:
        # read ini file
        config = ConfigObj(access_key_ini_file)
        if not config.get("dns_aliyun_key_id") or not config.get("dns_aliyun_key_secret"):
            raise click.UsageError(f"invalid ini file {access_key_ini_file}, please check")
        ctx.obj = Aliyun(
            config.get("dns_aliyun_key_id"),
            config.get("dns_aliyun_key_secret"),
        )
    else:
        raise click.UsageError("access-key-id and access-key-secret or access-key-ini-file is required")


@cli.command()
@click.option("--cdn", is_flag=True, help="show CDN domains")
@click.option("--live", is_flag=True, help="show live domains")
@pass_aliyun
def list_domains(aliyun: Aliyun, cdn: bool, live: bool) -> None:
    """
    show domains and their certificates
    """
    if not cdn and not live:
        raise click.UsageError("please specify --cdn or --live")
    if cdn:
        for d, certs in aliyun.iter_cdn_domains():
            subpanels = []
            for c in certs:
                g = Table.grid()
                g.add_column(min_width=15, justify="left", style="dim")
                g.add_column(justify="left")
                g.add_row("id", c.cert_id)
                g.add_row("domain", c.cert_domain_name)
                g.add_row("type", c.cert_type)
                g.add_row("status", c.status)
                g.add_row("life", c.cert_life)
                if c.cert_expire_time:
                    days_left = calc_left_days(c.cert_expire_time)
                    g.add_row(
                        "expired",
                        ("[bold red]TRUE[/]" if days_left < 0 else c.cert_expire_time + f" ({days_left} days left)"),
                    )
                subpanels.append(Panel(g, title=f"[bold blue]{c.cert_name}[/]", title_align="left"))
            cprint(
                Panel(
                    Group(*subpanels),
                    title=f"[bold green]{d.domain_name}[/]",
                    title_align="left",
                )
            )
    if live:
        for d, certs in aliyun.iter_live_domains():
            subpanels = []
            for c in certs:
                g = Table.grid()
                g.add_column(min_width=15, justify="left", style="dim")
                g.add_column(justify="left")
                g.add_row("domain", c.cert_domain_name)
                g.add_row("type", c.cert_type)
                g.add_row("status", c.status)
                g.add_row("life", c.cert_life)
                if c.cert_expire_time:
                    days_left = calc_left_days(c.cert_expire_time)
                    g.add_row(
                        "expired",
                        ("[bold red]TRUE[/]" if days_left < 0 else c.cert_expire_time + f" ({days_left} days left)"),
                    )
                subpanels.append(Panel(g, title=f"[bold blue]{c.cert_name}[/]", title_align="left"))
            cprint(
                Panel(
                    Group(*subpanels),
                    title=f"[bold green]{d.domain_name}[/]",
                    title_align="left",
                )
            )


@cli.command()
@pass_aliyun
def list_certs(aliyun: Aliyun) -> None:
    """
    show all uploaded certificates in aliyun CAS
    """
    for c in aliyun.iter_certs():
        days_left = "N/A"
        if not c.expired and c.end_date:
            days_left = (dateutil.parser.isoparse(c.end_date) - datetime.now()).days
        g = Table.grid()
        g.add_column(min_width=15, justify="left", style="dim")
        g.add_column(justify="left")
        g.add_row("name", c.name)
        g.add_row("common name", c.common_name)
        g.add_row("SANs", c.sans)
        g.add_row("status", c.status)
        g.add_row("issuer", c.issuer)
        g.add_row("start", c.start_date)
        g.add_row(
            "expired",
            ("[bold red]TRUE[/]" if c.expired else str(c.end_date) + f" ({days_left} days left)"),
        )
        cprint(Panel(g, title=f"[bold green]{c.certificate_id}[/]", title_align="left"))


@cli.command()
@click.option("--cert-id", required=True, type=int, help="certificate id")
@pass_aliyun
def get_cert(aliyun: Aliyun, cert_id: int) -> None:
    """
    get certificate detail by id
    """
    c = aliyun.get_cert_by_id(cert_id)
    days_left = "N/A"
    if not c.expired and c.end_date:
        days_left = (dateutil.parser.isoparse(c.end_date) - datetime.now()).days
    g = Table.grid()
    g.add_column(min_width=15, justify="left", style="dim")
    g.add_column(justify="left")
    g.add_row("name", c.name)
    g.add_row("common name", c.common)
    g.add_row("SANs", c.sans)
    g.add_row("issuer", c.issuer)
    g.add_row("start", c.start_date)
    g.add_row(
        "expired",
        ("[bold red]TRUE[/]" if c.expired else str(c.end_date) + f" ({days_left} days left)"),
    )
    cprint(Panel(g, title=f"[bold green]{c.id}[/]", title_align="left"))


@cli.command()
@click.argument("full-chain", type=click.File("r"), required=True)
@click.argument("private-key", type=click.File("r"), required=True)
@click.option("--domain", required=True, help="domain name")
@pass_aliyun
def upload_cert(aliyun: Aliyun, full_chain: TextIO, private_key: TextIO, domain: str) -> None:
    """
    upload certificate to aliyun CAS
    """
    cert = aliyun.upload_cert(domain, full_chain.read(), private_key.read())
    cprint(f"cert [bold green]{cert.id}[/] uploaded for {domain}")


@cli.command()
@click.option("--cert-id", required=True, type=int, help="certificate id")
@pass_aliyun
def delete_cert(aliyun: Aliyun, cert_id: int) -> None:
    """
    delete certificate from aliyun CAS
    """
    aliyun.delete_cert(cert_id)
    cprint(f"cert [bold red]{cert_id}[/] deleted")


@cli.command()
@click.option("--cert-id", required=True, type=int, help="certificate id")
@click.option("--domain", required=True, type=str, help="domain name")
@click.option(
    "--service",
    type=click.Choice(["cdn", "live"], case_sensitive=False),
    help="aliyun service type",
)
@pass_aliyun
def set_cert(aliyun: Aliyun, cert_id: int, domain: str, service: str) -> None:
    """
    set certificate for aliyun domain
    """
    if service == "cdn":
        c, cdn = aliyun.set_cert_for_cdn_domain(cert_id, domain)
        if not c:
            raise click.ClickException(f"certificate {cert_id} not found")
        if not cdn:
            raise click.ClickException(f"CDN domain {domain} not found")
        cprint(f"cert [bold green]{cert_id}[/] set for CDN domain [bold green]{cdn.domain_name}[/]")
    elif service == "live":
        c, live = aliyun.set_cert_for_live_domain(cert_id, domain)
        if not c:
            raise click.ClickException(f"certificate {cert_id} not found")
        if not live:
            raise click.ClickException(f"live domain {domain} not found")
        cprint(f"cert [bold green]{cert_id}[/] set for live domain [bold green]{live.domain_name}[/]")
    else:
        raise click.UsageError("please specify --service cdn or --service live")


@cli.command()
@click.option("--cert-id", required=True, type=int, help="certificate id")
@click.option("--cdn", is_flag=True, help="replace certificates of CDN domains")
@click.option("--live", is_flag=True, help="replace certificates of live domains")
@pass_aliyun
def replace_cert(aliyun: Aliyun, cert_id: int, cdn: bool, live: bool) -> None:
    """
    replace certificate for all domains which using certificate of the same common name
    """
    if not cdn and not live:
        raise click.UsageError("please specify --cdn or --live")
    if cdn:
        aliyun.replace_cert_for_all_matching_cdn_domains(cert_id)
    if live:
        aliyun.replace_cert_for_all_matching_live_domains(cert_id)


@cli.command()
@click.option(
    "--cert-path",
    envvar="RENEWED_LINEAGE",
    required=True,
    help="path to directory containing fullchain.pem and privkey.pem",
)
@click.option(
    "--renewed_domains",
    envvar="RENEWED_DOMAINS",
    type=RenewedDomains(),
    required=True,
    help="renewed domain names split by space",
)
@click.option("--cdn", is_flag=True, help="replace certificates of CDN domains")
@click.option("--live", is_flag=True, help="replace certificates of live domains")
@click.option("--delete-old-cert", is_flag=True, help="delete old certificate after deployment")
@pass_aliyun
def certbot_deploy_hook(
    aliyun: Aliyun,
    cert_path,
    renewed_domains: List[str],
    cdn: bool,
    live: bool,
    delete_old_cert: bool,
) -> None:
    """
    deploy hook for certbot

    please check "--deploy-hook DEPLOY_HOOK" in
    https://eff-certbot.readthedocs.io/en/stable/using.html

    """
    if not cdn and not live:
        raise click.UsageError("please specify --cdn or --live")
    with open(os.path.join(cert_path, "fullchain.pem"), "r") as f:
        full_chain = f.read()
    with open(os.path.join(cert_path, "privkey.pem"), "r") as f:
        private_key = f.read()
    cert = aliyun.upload_cert(renewed_domains[0], full_chain, private_key)
    if not isinstance(cert.id, int):
        raise click.ClickException(f"failed to upload certificate for <{' '.join(d for d in renewed_domains)}>")
    log.info(f"certificate for <{' '.join(d for d in renewed_domains)}> uploaded, id: <{cert.id}>")
    cert_id_to_delete = set()
    has_error = False
    if cdn:
        _, _, old_cert_ids, errors = aliyun.replace_cert_for_all_matching_cdn_domains(cert.id)
        cert_id_to_delete.update(old_cert_ids)
        if errors:
            has_error = True
    if live:
        _, _, old_cert_ids, errors = aliyun.replace_cert_for_all_matching_live_domains(cert.id)
        cert_id_to_delete.update(old_cert_ids)
        if errors:
            has_error = True
    if not has_error and delete_old_cert:
        for cert_id in cert_id_to_delete:
            aliyun.delete_cert(cert_id)
            log.info(f"deleted old certificate <{cert_id}>")


def calc_left_days(dts: str) -> int:
    if not dts:
        return -1
    dt = dateutil.parser.isoparse(dts)
    return (dt - datetime.now(tz=timezone.utc)).days


if __name__ == "__main__":
    cli()



from __future__ import annotations

from typing import List, Tuple, Set
from collections.abc import Generator
from datetime import datetime
from alibabacloud_cdn20180510.client import Client as Cdn20180510Client
from alibabacloud_live20161101.client import Client as live20161101Client
from alibabacloud_cas20200407.client import Client as cas20200407Client
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_cdn20180510 import models as cdn_20180510_models
from alibabacloud_live20161101 import models as live_20161101_models
from alibabacloud_cas20200407 import models as cas_20200407_models

import logging

log = logging.getLogger(__name__)


class Aliyun:
    def __init__(self, access_key_id, access_key_secret) -> None:
        self._cdn_client = Cdn20180510Client(
            open_api_models.Config(
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
                # Endpoint https://api.aliyun.com/product/Cdn
                endpoint="cdn.aliyuncs.com",
            )
        )
        self._live_client = live20161101Client(
            open_api_models.Config(
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
                # Endpoint https://api.aliyun.com/product/live
                endpoint="live.aliyuncs.com",
            )
        )
        self._cas_client = cas20200407Client(
            open_api_models.Config(
                access_key_id=access_key_id,
                access_key_secret=access_key_secret,
                # Endpoint https://api.aliyun.com/product/cas
                endpoint="cas.aliyuncs.com",
            )
        )

    def iter_certs(
        self,
    ) -> Generator[
        cas_20200407_models.ListUserCertificateOrderResponseBodyCertificateOrderList,
        None,
        None,
    ]:
        certs = self._cas_client.list_user_certificate_order(
            cas_20200407_models.ListUserCertificateOrderRequest(order_type="UPLOAD", show_size=100)
        ).body.certificate_order_list
        for c in certs:
            yield c

    def get_cert_by_id(self, cert_id: int) -> cas_20200407_models.GetUserCertificateDetailResponseBody:
        return self._cas_client.get_user_certificate_detail(
            cas_20200407_models.GetUserCertificateDetailRequest(cert_id=cert_id)
        ).body

    def upload_cert(
        self, domain_name: str, full_chain: str, private_key: str
    ) -> cas_20200407_models.GetUserCertificateDetailResponseBody:
        cert_name = domain_name.replace(".", "_") + datetime.now().strftime("_%Y%m%dT%H%M%S")
        cert_id = self._cas_client.upload_user_certificate(
            cas_20200407_models.UploadUserCertificateRequest(
                name=cert_name,
                cert=full_chain,
                key=private_key,
            )
        ).body.cert_id
        if isinstance(cert_id, int):
            return self.get_cert_by_id(cert_id)
        else:
            raise Exception(f"Failed to upload certificate")

    def set_cert_for_cdn_domain(self, cert_id: int, domain_name: str) -> Tuple[
        cas_20200407_models.GetUserCertificateDetailResponseBody | None,
        cdn_20180510_models.DescribeUserDomainsResponseBodyDomainsPageData | None,
    ]:
        cert = self.get_cert_by_id(cert_id)
        if not cert:
            raise Exception(f"Failed to get certificate {cert_id}")
        for d in self._cdn_client.describe_user_domains(
            cdn_20180510_models.DescribeUserDomainsRequest(
                page_number=1,
                page_size=50,
            ),
        ).body.domains.page_data:
            if d.domain_name == domain_name:
                self._cdn_client.set_cdn_domain_sslcertificate(
                    cdn_20180510_models.SetCdnDomainSSLCertificateRequest(
                        cert_id=cert_id,
                        cert_type="cas",
                        cert_name=str(cert.name),
                        domain_name=domain_name,
                        sslprotocol="on",
                    )
                )
                return cert, d
        return cert, None

    def set_cert_for_live_domain(self, cert_id: int, domain_name: str) -> Tuple[
        cas_20200407_models.GetUserCertificateDetailResponseBody | None,
        live_20161101_models.DescribeLiveUserDomainsResponseBodyDomainsPageData | None,
    ]:
        cert = self.get_cert_by_id(cert_id)
        if not cert:
            raise Exception(f"Failed to get certificate {cert_id}")
        for d in self._live_client.describe_live_user_domains(
            live_20161101_models.DescribeLiveUserDomainsRequest(
                page_number=1,
                page_size=50,
            )
        ).body.domains.page_data:
            if d.domain_name == domain_name:
                self._live_client.set_live_domain_certificate(
                    live_20161101_models.SetLiveDomainCertificateRequest(
                        cert_name=str(cert.name),
                        cert_type="cas",
                        domain_name=domain_name,
                        sslprotocol="on",
                    )
                )
                return cert, d
        return cert, None

    def replace_cert_for_all_matching_cdn_domains(self, new_cert_id: int) -> Tuple[
        cas_20200407_models.GetUserCertificateDetailResponseBody | None,
        List[cdn_20180510_models.DescribeUserDomainsResponseBodyDomainsPageData],
        Set[int],
        List[Exception],
    ]:
        new_cert = self.get_cert_by_id(new_cert_id)
        if not new_cert:
            raise Exception(f"Failed to get certificate {new_cert_id}")
        old_certs_by_id = {c.certificate_id: c for c in self.iter_certs()}
        replaced_domains = []
        old_cert_ids = set()
        errors = []
        for d, old_certs in self.iter_cdn_domains():
            try:
                to_set = False
                for old_cert in old_certs:
                    if not isinstance(old_cert.cert_id, str):
                        raise Exception(f"Invalid cert_id: {old_cert.cert_id}")
                    old_cert_id = int(old_cert.cert_id)
                    if old_cert_id in old_cert_ids:
                        to_set = True
                        break
                    oc = old_certs_by_id.get(old_cert_id)
                    if oc and oc.common_name == new_cert.common and oc.certificate_id != new_cert_id:
                        old_cert_ids.add(oc.certificate_id)
                        to_set = True
                        break
                if to_set:
                    self._cdn_client.set_cdn_domain_sslcertificate(
                        cdn_20180510_models.SetCdnDomainSSLCertificateRequest(
                            cert_id=new_cert_id,
                            cert_type="cas",
                            cert_name=str(new_cert.name),
                            domain_name=str(d.domain_name),
                            sslprotocol="on",
                        )
                    )
                    replaced_domains.append(d)
                    log.info(f"certificate <{new_cert_id}> set for CDN domain <{d.domain_name}>")
            except Exception as e:
                log.exception(e)
                errors.append(e)
        return new_cert, replaced_domains, old_cert_ids, errors

    def replace_cert_for_all_matching_live_domains(self, new_cert_id: int) -> Tuple[
        cas_20200407_models.GetUserCertificateDetailResponseBody | None,
        List[live_20161101_models.DescribeLiveUserDomainsResponseBodyDomainsPageData],
        Set[int],
        List[Exception],
    ]:
        new_cert = self.get_cert_by_id(new_cert_id)
        if not new_cert:
            raise Exception(f"Failed to get certificate {new_cert_id}")
        old_certs_by_id = {c.certificate_id: c for c in self.iter_certs()}
        old_certs_by_name = {c.name: c for c in old_certs_by_id.values()}
        replaced_domains = []
        old_cert_ids = set()
        errors = []
        for d, old_certs in self.iter_live_domains():
            try:
                to_set = False
                for old_cert in old_certs:
                    oc = old_certs_by_name.get(old_cert.cert_name)
                    if oc and oc.certificate_id in old_cert_ids:
                        to_set = True
                        break
                    if oc and oc.common_name == new_cert.common and oc.certificate_id != new_cert_id:
                        old_cert_ids.add(oc.certificate_id)
                        to_set = True
                        break
                if to_set:
                    self._live_client.set_live_domain_certificate(
                        live_20161101_models.SetLiveDomainCertificateRequest(
                            cert_name=str(new_cert.name),
                            cert_type="cas",
                            domain_name=str(d.domain_name),
                            sslprotocol="on",
                        )
                    )
                    replaced_domains.append(d)
                    log.info(f"certificate <{new_cert_id}> set for Live domain <{d.domain_name}>")
            except Exception as e:
                log.exception(e)
                errors.append(e)
        return new_cert, replaced_domains, old_cert_ids, errors

    def delete_cert(self, cert_id: int) -> None:
        self._cas_client.delete_user_certificate(cas_20200407_models.DeleteUserCertificateRequest(cert_id=cert_id))

    def iter_cdn_domains(
        self,
    ) -> Generator[
        Tuple[
            cdn_20180510_models.DescribeUserDomainsResponseBodyDomainsPageData,
            List[cdn_20180510_models.DescribeDomainCertificateInfoResponseBodyCertInfosCertInfo],
        ],
        None,
        None,
    ]:
        for d in self._cdn_client.describe_user_domains(
            cdn_20180510_models.DescribeUserDomainsRequest(
                page_number=1,
                page_size=50,
            ),
        ).body.domains.page_data:
            if d.ssl_protocol == "off":
                continue
            certs = self._cdn_client.describe_domain_certificate_info(
                cdn_20180510_models.DescribeDomainCertificateInfoRequest(domain_name=str(d.domain_name))
            ).body.cert_infos.cert_info
            yield d, certs

    def iter_live_domains(
        self,
    ) -> Generator[
        Tuple[
            live_20161101_models.DescribeLiveUserDomainsResponseBodyDomainsPageData,
            List[live_20161101_models.DescribeLiveDomainCertificateInfoResponseBodyCertInfosCertInfo],
        ],
        None,
        None,
    ]:
        for d in self._live_client.describe_live_user_domains(
            live_20161101_models.DescribeLiveUserDomainsRequest(
                page_number=1,
                page_size=50,
            )
        ).body.domains.page_data:
            certs = self._live_client.describe_live_domain_certificate_info(
                live_20161101_models.DescribeLiveDomainCertificateInfoRequest(domain_name=str(d.domain_name))
            ).body.cert_infos.cert_info
            yield d, certs
